/*
 * Feed parser backend
 *
 * In this module you'll be implementing the feed parser backend.
 * This code will not be Unity-specific, and it will vary depending
 * on the type of data returned by your source. Common feed types
 * returned when doing search queries to sources are JSON, RSS, XML.
 *
 * In our particular case, it's an RSS feed parser.
 *
 * You'll simply need to implement a parser for the type of feed your
 * source returns and send the results back to Unity via the get_results()
 * function.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mrss.h>
#include <glib.h>
#include "openclipart-parser.h"

/*
 * This is the URI against which you'll be submitting your
 * search query from the Dash.
 */
#define BASE_URI "http://openclipart.org/api/search/?query="

/**
 * @brief Cleans up (frees the memory allocated memory)
 *        of the given results structure
 * @param data Structure that contains the data to free
 */
void
result_cleanup(gpointer data) {
    result_t *result = (result_t *)data;
    if (result->link) {
        free(result->link);
    }
    if (result->icon_url) {
        free(result->icon_url);
    }
    if (result->title) {
        free(result->title);
    }
    if (result->description) {
        free(result->description);
    }
    if (result->creation_date) {
        free(result->creation_date);
    }
    if (result->author) {
        free(result->author);
    }
}

/**
 * Returns a value for a given tag and attribute. If attr_name is null,
 * then it returns the tag value, otherwise it returns the attribute value.
 * Returns NULL on any error or unfound tag/attribute.
 *
 * XXX - it would be more efficient to collect stuff from this loop all at
 * once.
 *
 * @brief Returns a value for a given tag and attribute
 * @param tag Tag id to get the value for
 * @param tag_name The name of the tag
 * @param attr_name The name of the attribute
 * @return Value for the given tag or attribute
 */
char *
get_other_tags_attribute(mrss_tag_t *tag, const char *tag_name, const char *attr_name) {
    mrss_attribute_t *attribute;

    if ((tag_name == NULL) && (attr_name == NULL)) {
        return NULL;
    }

    while (tag) {
        if (!strcmp(tag->name, tag_name)) {
            if (attr_name == NULL) {
                return tag->value;
            }
            for (attribute = tag->attributes; attribute;
                attribute = attribute->next) {
                if (!strcmp(attribute->name, attr_name)) {
                    return attribute->value;
                }
            }
        }
        tag = tag->next;
    }
    return NULL;
}

/**
 * @brief get_results Get and parse the results from a search query
 * @param search_term String submitted as the search term
 * @return Search results
 */
GSList *
get_results(const char *search_term) {
    mrss_t *data;
    mrss_error_t ret;
    mrss_item_t *item;
    CURLcode code;
    GString *url = NULL;
    GSList *results = NULL;
    result_t *result = NULL;
    char *tmp = NULL;

    /* Check if an actual search term was submitted, return otherwise */
    if (search_term == NULL) {
        g_warning("get_results: search_term cannot be null");
        return results;
    }

    /* Construct the full search query */
    url = g_string_new(BASE_URI);
    g_string_append(url, search_term);
    g_debug("searching %s", url->str);
    ret = mrss_parse_url_with_options_and_error(url->str, &data, NULL, &code);
    g_string_free(url,TRUE);

    if (ret) {
        g_warning("MRSS return error: %s\n", ret == MRSS_ERR_DOWNLOAD ? mrss_curl_strerror (code) : mrss_strerror (ret));
        return results;
    }

    /*
     * Parse the results from the RSS feed returned by a search query submitted to openclipart.org
     * See "Clipart Search" in http://openclipart.org/developers
     */
    item = data->item;
    while (item) {
        result = (result_t*)malloc(sizeof(result_t));
        bzero((result_t*)result, sizeof(result_t));
        if (item->link) {
            result->link = (char *)malloc(strlen(item->link)+1);
            strcpy(result->link,item->link);
        }
        if (item->title) {
            result->title = (char *)malloc(strlen(item->title)+1);
            strcpy(result->title,item->title);
        }
        if (item->description) {
            result->description = (char *)malloc(strlen(item->description)+1);
            strcpy(result->description,item->description);
        }
        if (item->pubDate) {
            result->creation_date = (char *)malloc(strlen(item->pubDate)+1);
            strcpy(result->creation_date,item->pubDate);
        }
        tmp = get_other_tags_attribute(item->other_tags, "thumbnail", "url");
        if (tmp) {
            result->icon_url = (char *)malloc(strlen(tmp)+1);
            strcpy(result->icon_url,tmp);
        }
        tmp = get_other_tags_attribute(item->other_tags, "creator", NULL);
        if (tmp) {
            result->author = (char *)malloc(strlen(tmp)+1);
            strcpy(result->author,tmp);
        }
        results = g_slist_append(results, result);
        item = item->next;
    }
    mrss_free (data);
    return results;
}
