/**
 * Instance of the Ubuntu UI used to provide an interface to the
 * Ubuntu SDK HTML5 theme
 * @type {UbuntuUI}
 */
var UI = new UbuntuUI();

// This is the event that fires when Cordova is fully loaded
document.addEventListener("deviceready", onDeviceReady, false);

/**
 * The name of the key where scan code history will
 * be stored via HTML5 local storage
 * @type {string}
 */
var SCAN_HISTORY_KEY='scan-code-history-key';

/**
 * Appends a successfully scanned QR code to history.
 * @param {string} data Scanned data.
 */
function appendScanToHistory(data) {

    // Check if HTML5 local storage is supported and if there
    // was any data passed in
    if ( ! window.localStorage || ! data) {
        return;
    }

    // If it's the first time we're using the history,
    // create it with empty content
    if ( ! window.localStorage.getItem(SCAN_HISTORY_KEY)) {
        window.localStorage.setItem(SCAN_HISTORY_KEY, JSON.stringify([]));
    }

    /**
     * Appends a successfully scanned QR code and its geolocation to the history.
     * @param {Coordinates} position Geolocation where the QR code was scanned.
     */
    function saveScanResultTakenAtPosition(position) {
        // Retrieve the scan history
        var history = JSON.parse(window.localStorage.getItem(SCAN_HISTORY_KEY));

        // Append the scanned code's data, its date and location to the history
        history.push({data: data, time: new Date(), location: position.coords});
        window.localStorage.setItem(SCAN_HISTORY_KEY, JSON.stringify(history));
    };

    // We disable the geolocation API for now, as it's still under development
    // for Ubuntu Touch

    /*
    function geolocationError(error) {
        console.log(error);
    };

    navigator.geolocation.getCurrentPosition(
	saveScanResultTakenAtPosition,
        geolocationError,
        { maximumAge: 3000, timeout: 5000, enableHighAccuracy: true });
   */

    // ... instead we just save the result with a dummy position (guess which one!)
    saveScanResultTakenAtPosition({coords: {latitude: '34.943742', longitude:'136.888269'}});
};

/**
 * Shows the result of the scanned QR code
 * @param {string} data The scanned QR code data.
 */
function displayScannedResults(data) {
    document.getElementById('scan-results').innerHTML = data;
};

/**
 * Scans the QR code
 */
function doScanCode() {
    // Error strings returned by the qrcode js library, so that
    //  we can try (even with false positives) to identify scanning
    //  errors & user cancelled operations

    var ERROR_STRING = 'error decoding QR Code';
    var CANCELLED_SCAN = 'canceled';

    var onQrScanningComplete = function(data) {

        // We use the Notification API to vibrate
        navigator.notification.vibrate(1000);

        // Show the result of the scan
        displayScannedResults(data);

        // Hide the Scanning progress dialog
        // now that the scan is complete
        UI.dialog('Scanning').hide();

        // Append the scanned data to history
        if (data != ERROR_STRING && data != CANCELLED_SCAN)
            appendScanToHistory(data);
    };


    /**
     * Once a picture is taken, this callback decodes the image
     * containing a QR code
     * @param {string} data Base64-encoded JPEG image
     */
    var onPictureTaken = function (data) {
        qrcode.callback = onQrScanningComplete;

        // Show the Scanning dialog to indicate
        // progress
        UI.dialog('Scanning').show();

        // Use the QR library to decode the image data
        // in base64 encoding
        qrcode.decode("data:image/jpeg;base64," + data);
    };

    /**
     * Once a picture is taken, this callback logs any
     * errors to the console
     * @param {string} message Scanning error message
     */
    var onPictureErrors = function(message) {
        console.log('Scan Code error: ' + message);
    };

    // Use the Camera API to take a picture using the
    // web cam
    navigator.camera.getPicture(onPictureTaken,
                                onPictureErrors,
                                { quality: 100,
                                  targetWidth: 300,
                                  targetHeight: 300,
                                  sourceType: Camera.PictureSourceType.CAMERA,
                                  encodingType: Camera.EncodingType.JPEG,
                                  destinationType: Camera.DestinationType.DATA_URL });
};

/**
 * Event handler that shows the history list on the UI
 */
function onHistoryList() {

    // Check if there is a scan history
    if ( ! window.localStorage.getItem(SCAN_HISTORY_KEY))
        return;

    // Get a copy of the history
    var history = JSON.parse(window.localStorage.getItem(SCAN_HISTORY_KEY));

    // Clear all history items on the UI
    UI.list('#history-items-list').removeAllItems();

    // Append each history item to a list for display purposes
    // Note that we add an onClick handler that will take the viewer
    // to a Google maps page showing the location where the QR code was taken
    history.forEach(function (element) {
        UI.list('#history-items-list').append(element.data,
                                              // the label for this item
                                              element.time,
                                              // no specific 'id'
                                              null,
                                              // onclick handler
                                              function (targetNode, location) {
                                              // We use the Cordova InAppBrowser 'API'
                                                  window.open('https://maps.google.ca/maps?safe=off&q=' + location.latitude + '+' + location.longitude);
                                              },
                                              element.location);
    });
};

/**
 * Entry point to the app. It initializes the Ubuntu SDK HTML5 theme
 * and connects events to handlers
 */
function onDeviceReady() {

    /***** Initialization *****/

    // Initialize the Ubuntu SDK HTML5 theme
    UI.init();

    // Set up the app by pushing the main view
    UI.pagestack.push("main-page");

    /***** Connecting events *****/

    // On clicking the scan button, show the scan page
    UI.button('scan').click(function (e) {
        UI.pagestack.push('scan-page',
                          {subtitle: 'Scan Code'});
        displayScannedResults('');
    });

    // On clicking the history button, show the history page
    UI.button('history').click(function (e) {
        UI.pagestack.push('history-page',
                          {subtitle: 'Scan History'});
        onHistoryList();
    });

    // On clicking the info button, show the info page
    UI.button('info').click(function (e) {
        UI.pagestack.push('info-page',
                          {subtitle: 'Application Info'});
    });

    // On clicking the do-scan button, start the scanning process
    UI.button('do-scan').click(function () {
        doScanCode();
    });
};

