#include "openclipart-query.h"
#include <unity/scopes/Annotation.h>
#include <unity/scopes/CategorisedResult.h>
#include <unity/scopes/CategoryRenderer.h>
#include <unity/scopes/QueryBase.h>
#include <unity/scopes/SearchReply.h>

#include <QDebug>
#include <QJsonArray>
#include <QJsonDocument>
#include <QJsonObject>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QNetworkRequest>
#include <QUrl>
#include <QCoreApplication>

using namespace unity::scopes;
//Create a JSON string to be used tro create a category renderer - uses grid layout
std::string CR_GRID = R"(
    {
        "schema-version" : 1,
        "template" : {
            "category-layout" : "grid",
            "card-size": "small"
        },
        "components" : {
            "title" : "title",
            "art" : {
                "field": "art",
                "aspect-ratio": 1.6,
                "fill-mode": "fit"
            }
        }
    }
)";

//Create a JSON string to be used tro create a category renderer - uses carousel layout
std::string CR_CAROUSEL = R"(
    {
        "schema-version" : 1,
        "template" : {
            "category-layout" : "carousel",
            "card-size": "small"
        },
        "components" : {
            "title" : "title",
            "art" : {
                "field": "art",
                "aspect-ratio": 1.6,
                "fill-mode": "fit"
            }
        }
    }
)";

//Create a URI to be used in the web API query
const QString BASE_URI = "http://openclipart.org/search/json/?query=%1";

OpenclipartQuery::OpenclipartQuery(CannedQuery const& query, SearchMetadata const& metadata) :
    SearchQueryBase(query, metadata)
{
}

OpenclipartQuery::~OpenclipartQuery()
{
}

void OpenclipartQuery::cancelled()
{
    // TODO: it would be nice to stop QEventLoop that's used in run() here,
    // but let's keep it simple for this example.
}

void OpenclipartQuery::run(unity::scopes::SearchReplyProxy const& reply)
{
    /* This is where the actual processing of the current search query takes place.
     * It's where you may want to query a local or remote data source for results
     * matching the query.*/

    /* We're now registering (creating) two new categoryies, one with grid layout, the other wiht carousel.
     * Categories can be created at any point
     * during query processing inside the run method, but it's recommended
     * to create them as soon as possible (ideally as soon as they are known to the scope) */
    CategoryRenderer rdrGrid(CR_GRID);
    CategoryRenderer rdrCarousel(CR_CAROUSEL);

    auto catGrid = reply->register_category("openclipartgrid", "OpenclipartGrid", "", rdrGrid);
    auto catCar = reply->register_category("openclipartcarousel", "OpenclipartCarousel", "", rdrCarousel);

    // Generate a network request to the OpenClipArt server and parse the result
    QEventLoop loop;

    QNetworkAccessManager manager;
    QObject::connect(&manager, SIGNAL(finished(QNetworkReply*)), &loop, SLOT(quit()));
    QObject::connect(&manager, &QNetworkAccessManager::finished,
            [reply, catCar, catGrid, this](QNetworkReply *msg){
                QByteArray data = msg->readAll();
                QJsonParseError err;
                QJsonDocument doc = QJsonDocument::fromJson(data, &err);

                if (err.error != QJsonParseError::NoError) {
                    qCritical() << "Failed to parse server data: " << err.errorString();
                } else {
                    // Find the "payload" array of results
                    QJsonObject obj = doc.object();
                    QJsonArray results = obj["payload"].toArray();

                    // for demo purposes, we we alternate between grid and carousel, so
                    // create a pointer to a Category and switch it between our two cateogries
                    // for each result
                    const std::shared_ptr<const Category> * cat;

                    bool grid = false;
                    //loop through results of our web query with each result called 'result'
                    for(const auto &result : results) {
                        if ( grid ) {
                          cat = &catGrid;
                          grid = false;
                        } else {
                          grid = true;
                          cat = &catCar;
                        }
                        //create our categorized result using our pointer, which is either to out
                        //grid or our carousel Category
                        CategorisedResult catres((*cat));

                        //treat result as Q JSON
                        QJsonObject resJ = result.toObject();

                        // load up vars with from result
                        auto title = resJ["title"].toString();
                        auto uploader = resJ["uploader"].toString();
                        auto artist = "Artist: " + resJ["drawn_by"].toString();
                        auto uri = resJ["detail_link"].toString();
                        auto image = resJ["svg"].toObject()["png_thumb"].toString();
                        auto desc = resJ["description"].toString();

                        //set our CateogroisedResult object with out searchresults values 
                        catres.set_uri(uri.toStdString());
                        catres.set_dnd_uri(uri.toStdString());
                        catres.set_title(title.toStdString());
                        catres.set_art(image.toStdString());
                        //Here we add a non-standard key/value which therefore has no convenience method
                        //Note that such non-standard keys do not display in the Results display phase.
                        //They may be displayed in the Preview phase. See openclipart-preview.cpp and
                        //search for 'description' to see how
                        catres["description"] = Variant(desc.toStdString());
                        catres["artist"] = Variant(artist.toStdString());

                        //push the categorized result to the client
                        if (!reply->push(catres)) {
                            break; // false from push() means search waas cancelled
                        }
                    }
                }
            }
            );

    // The query is the search string and was passed to this Query object's constructor by the client
    // Empty search string yields no results with openclipart API.
    // Let's set the query string to "cat" if empty so the scope displays results at first launch
    // in demo/devel mode
    if ( query_.empty() ) {
      query_ = "cat";
    }
    QString queryUri = BASE_URI.arg(query_.c_str());
    manager.get(QNetworkRequest(QUrl(queryUri)));
    loop.exec();
}
